=== modified file 'NEWS'
--- NEWS	2008-07-25 21:11:59 +0000
+++ NEWS	2008-06-22 23:47:59 +0000
@@ -4,9 +4,6 @@
 1.6 [unreleased]
 ----------------
 
-    - Remove the .py extension requiered by the Debian Policy.
-      (Jelmer Vernooij)
-
     - Loggerhead is no longer a TurboGears application, but rather a
       WSGI application built using Paste (see http://wsgi.org/ and
       http://pythonpaste.org/ for more about WSGI and Paste). XXX more.
@@ -26,9 +23,7 @@
       memory consumption of rendering by a factor of around 3 for
       large pages over the old Kid templates.
 
-    - Loggerhead's poorly performing text index was disabled. bzr-search
-      is now used if the plugin is installed and a text index is present
-      on the branch being viewed. (Martin Albisetti, Robert Collins).
+    - Loggerhead's poorly performing text index was disabled.
 
     - Loggerhead no longer depends on bzrlib functions deprecated in
       Bazaar 1.5 and removed in 1.6 (Martin Albisetti).

=== modified file 'loggerhead/apps/__init__.py'
--- loggerhead/apps/__init__.py	2008-07-01 04:01:31 +0000
+++ loggerhead/apps/__init__.py	2008-06-18 07:48:17 +0000
@@ -1,5 +1,4 @@
-"""WSGI applications for serving Bazaar branches."""
-
+#
 import os
 
 from paste import urlparser, fileapp

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2008-07-09 08:39:59 +0000
+++ loggerhead/apps/branch.py	2008-06-24 17:12:45 +0000
@@ -1,15 +1,12 @@
-"""The WSGI application for serving a Bazaar branch."""
-
 import logging
 import urllib
 
-import bzrlib.branch
-import bzrlib.lru_cache
-
 from paste import request
 from paste import httpexceptions
+from paste.wsgiwrappers import WSGIRequest, WSGIResponse
 
 from loggerhead.apps import static_app
+from loggerhead.changecache import FileChangeCache
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.annotate_ui import AnnotateUI
@@ -20,33 +17,27 @@
 from loggerhead.history import History
 from loggerhead import util
 
+logging.basicConfig()
+logging.getLogger().setLevel(logging.DEBUG)
 
 class BranchWSGIApp(object):
 
-    def __init__(self, branch, friendly_name=None, config={}, graph_cache=None):
-        self.branch = branch
+    def __init__(self, branch_url, friendly_name=None, config={}):
+        self.branch_url = branch_url
+        self._history = None
         self._config = config
         self.friendly_name = friendly_name
-        self.log = logging.getLogger('loggerhead.%s' % (friendly_name,))
-        if graph_cache is None:
-            graph_cache = bzrlib.lru_cache.LRUCache()
-        self.graph_cache = graph_cache
+        self.log = logging.getLogger(friendly_name)
 
-    def get_history(self):
-        _history = History(self.branch, self.graph_cache)
-        cache_path = self._config.get('cachepath', None)
-        if cache_path is not None:
-            # Only import the cache if we're going to use it.
-            # This makes sqlite optional
-            try:
-                from loggerhead.changecache import FileChangeCache
-            except ImportError:
-                self.log.debug("Couldn't load python-sqlite,"
-                               " continuing without using a cache")
-            else:
-                _history.use_file_cache(
-                    FileChangeCache(_history, cache_path))
-        return _history
+    @property
+    def history(self):
+        if (self._history is None) or self._history.out_of_date():
+            self.log.debug('Reload branch history...')
+            _history = self._history = History.from_folder(self.branch_url)
+            cache_path = self._config.get('cachepath', None)
+            if cache_path is not None:
+                _history.use_file_cache(FileChangeCache(_history, cache_path))
+        return self._history
 
     def url(self, *args, **kw):
         if isinstance(args[0], list):
@@ -79,14 +70,17 @@
         }
 
     def last_updated(self):
-        h = self.get_history()
+        h = self.history
         change = h.get_changes([ h.last_revid ])[0]
         return change.date
 
     def branch_url(self):
-        return self.branch.get_config().get_user_option('public_branch')
+        return self.history.get_config().get_user_option('public_branch')
 
     def app(self, environ, start_response):
+        req = WSGIRequest(environ)
+        response = WSGIResponse()
+        response.headers['Content-Type'] = 'text/plain'
         self._url_base = environ['SCRIPT_NAME']
         self._static_url_base = environ.get('loggerhead.static.url')
         if self._static_url_base is None:
@@ -101,9 +95,6 @@
         cls = self.controllers_dict.get(path)
         if cls is None:
             raise httpexceptions.HTTPNotFound()
-        self.branch.lock_read()
-        try:
-            c = cls(self, self.get_history())
-            return c(environ, start_response)
-        finally:
-            self.branch.unlock()
+        c = cls(self)
+        c.default(req, response)
+        return response(environ, start_response)

=== modified file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2008-07-01 04:01:31 +0000
+++ loggerhead/apps/config.py	2008-06-23 01:03:34 +0000
@@ -1,15 +1,12 @@
-"""A server that uses a loggerhead.conf file.
+# A server that recreates (modulo cherrypy bugs :) the url parsing
+# from the old, loggerhead.conf approach.
 
-We recreate the branch discovery and url scheme of the old branchview
-code.  It's all a bit horrible really.
-"""
+# It's all a bit horrible really.
 
 import logging
 import os
 import posixpath
 
-import bzrlib.lru_cache
-
 from configobj import ConfigObj
 
 from paste.request import path_info_pop
@@ -26,23 +23,17 @@
 
 from loggerhead.history import is_branch
 
-class Project(object):
-    """A project contains the branches.
-
-    There is some complication because we don't want to hold on to the
-    branches when they are not being browsed."""
-
-    def __init__(self, name, config, root_config, graph_cache):
+class Project (object):
+    def __init__(self, name, config, root_config):
         self.name = name
         self.friendly_name = config.get('name', name)
         self.description = config.get('description', '')
         self.long_description = config.get('long_description', '')
         self._config = config
         self._root_config = root_config
-        self.graph_cache = graph_cache
 
-        self.view_names = []
-        self.view_data_by_name = {}
+        self.views = []
+        self.views_by_name = {}
         for view_name in config.sections:
             log.debug('Configuring (project %s) branch %s...', name, view_name)
             self._add_view(
@@ -69,8 +60,8 @@
             return
 
         # rebuild views:
-        self.view_names = []
         log.debug('Rescanning auto-folder for project %s ...', self.name)
+        self._views = []
         for folder in auto_list:
             view_name = os.path.basename(folder)
             log.debug('Auto-configuring (project %s) branch %s...', self.name, view_name)
@@ -86,120 +77,74 @@
             return posixpath.join(url, folder) + '/'
         return None
 
-    def _get_description(self, view, view_config, history):
+    def _get_description(self, view, view_config):
         description = view_config.get('description', None)
         if description is not None:
             return description
-        description = history._branch.get_config().get_user_option('description')
+        description = view.history._branch.get_config().get_user_option('description')
         return description
 
     def _add_view(self, view_name, view_config, folder):
-        b = bzrlib.branch.Branch.open(folder)
-        view = BranchWSGIApp(b, view_name, view_config, self.graph_cache)
-        b.lock_read()
-        try:
-            history = view.get_history()
-            friendly_name = view_config.get('branch_name', None)
+        view = BranchWSGIApp(folder, view_name, view_config)
+        friendly_name = view_config.get('branch_name', None)
+        if friendly_name is None:
+            friendly_name = view.history.get_config().get_nickname()
             if friendly_name is None:
-                friendly_name = history.get_config().get_nickname()
-                if friendly_name is None:
-                    friendly_name = view_name
-            self.view_data_by_name[view_name] = {
-                'branch_path': folder,
-                'args': (view_name, view_config, self.graph_cache),
-                'description': self._get_description(view, view_config, history),
-                '_src_folder': folder,
-                '_view_config': view_config,
-                'friendly_name': friendly_name,
-                'name': view_name,
-                }
-            branch_url = self._get_branch_url(view, view_config, view_name)
-            if branch_url is not None:
-                self.view_data_by_name[view_name]['branch_url'] = branch_url
-            self.view_names.append(view_name)
-        finally:
-            b.unlock()
-
-    def view_named(self, name):
-        view_data = self.view_data_by_name.get(name)
-        if view_data is None:
-            return None
-        view_data = view_data.copy()
-        branch_path = view_data.pop('branch_path')
-        args = view_data.pop('args')
-        b = bzrlib.branch.Branch.open(branch_path)
-        b.lock_read()
-        view = BranchWSGIApp(b, *args)
-        for k in view_data:
-            setattr(view, k, view_data[k])
-        return view
+                friendly_name = view_name
+        view.friendly_name = friendly_name
+        view.name = view_name
+        branch_url = self._get_branch_url(view, view_config, view_name)
+        if branch_url is not None:
+            view.branch_url = branch_url
+        view.description = self._get_description(view, view_config)
+        view._src_folder = folder
+        view._view_config = view_config
+        self.views.append(view)
+        self.views_by_name[view_name] = view
 
     def call(self, environ, start_response):
         segment = path_info_pop(environ)
         if not segment:
             raise httpexceptions.HTTPNotFound()
         else:
-            view = self.view_named(segment)
+            view = self.views_by_name.get(segment)
             if view is None:
                 raise httpexceptions.HTTPNotFound()
-            try:
-                return view.app(environ, start_response)
-            finally:
-                view.branch.unlock()
+            return view.app(environ, start_response)
 
 
 class Root(object):
-    """The root of the server -- renders as the browse view,
-    dispatches to Project above for each 'project'."""
 
     def __init__(self, config):
         self.projects = []
         self.config = config
         self.projects_by_name = {}
-        graph_cache = bzrlib.lru_cache.LRUCache()
         for project_name in self.config.sections:
             project = Project(
-                project_name, self.config[project_name], self.config, graph_cache)
+                project_name, self.config[project_name], self.config)
             self.projects.append(project)
             self.projects_by_name[project_name] = project
 
     def browse(self, response):
-        # This is insanely complicated because we want to open and
-        # lock all the branches, render the view and then unlock the
-        # branches again.
         for p in self.projects:
             p._recheck_auto_folders()
-        class branch(object):
+        class branch:
             @staticmethod
             def static_url(path):
                 return self._static_url_base + path
-        views_by_project = {}
-        all_views = []
-        try:
-            for p in self.projects:
-                views_by_project[p] = []
-                for vn in p.view_names:
-                    v = p.view_named(vn)
-                    all_views.append(v)
-                    views_by_project[p].append(v)
-            vals = {
-                'projects': self.projects,
-                'util': util,
-                'title': self.config.get('title', None),
-                'branch': branch,
-                'views_by_project': views_by_project,
-            }
-            vals.update(templatefunctions)
-            response.headers['Content-Type'] = 'text/html'
-            template = load_template('loggerhead.templates.browse')
-            template.expand_into(response, **vals)
-        finally:
-            for v in all_views:
-                v.branch.unlock()
+        vals = {
+            'projects': self.projects,
+            'util': util,
+            'title': self.config.get('title', None),
+            'branch': branch,
+        }
+        vals.update(templatefunctions)
+        response.headers['Content-Type'] = 'text/html'
+        template = load_template('loggerhead.templates.browse')
+        template.expand_into(response, **vals)
 
     def __call__(self, environ, start_response):
-        self._static_url_base = environ['loggerhead.static.url'] = \
-                                environ['SCRIPT_NAME']
+        self._static_url_base = environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
         segment = path_info_pop(environ)
         if segment is None:
             raise httpexceptions.HTTPMovedPermanently(

=== modified file 'loggerhead/apps/filesystem.py'
--- loggerhead/apps/filesystem.py	2008-07-01 04:01:31 +0000
+++ loggerhead/apps/filesystem.py	2008-06-18 12:27:04 +0000
@@ -1,10 +1,8 @@
-"""Serve branches at urls that mimic the file system layout."""
-
 import cgi
 import os
 import tempfile
 
-from bzrlib import branch, errors, lru_cache
+from bzrlib import branch, errors
 
 from paste.request import path_info_pop
 from paste.wsgiwrappers import WSGIRequest, WSGIResponse
@@ -13,71 +11,64 @@
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.apps import favicon_app, static_app
 
+
 sql_dir = tempfile.mkdtemp()
 
-
-class DirectoryListing(object):
-
-    def __init__(self, path):
-        self.path = path
-
-    def __call__(self, environ, start_response):
+class BranchesFromFileSystemServer(object):
+    def __init__(self, folder, root):
+        self.folder = folder
+        self.root = root
+
+    def directory_listing(self, path, environ, start_response):
         request = WSGIRequest(environ)
         response = WSGIResponse()
-        listing = [d for d in os.listdir(self.path) if not d.startswith('.')]
+        listing = [d for d in os.listdir(path) if not d.startswith('.')]
         response.headers['Content-Type'] = 'text/html'
         print >> response, '<html><body>'
         for d in sorted(listing):
-            if os.path.isdir(os.path.join(self.path, d)):
+            if os.path.isdir(os.path.join(path, d)):
                 d = cgi.escape(d)
                 print >> response, '<li><a href="%s/">%s</a></li>' % (d, d)
         print >> response, '</body></html>'
         return response(environ, start_response)
 
-
-class BranchesFromFileSystemServer(object):
-    def __init__(self, folder, root):
-        self.folder = folder
-        self.root = root
-
-    def app_for_branch(self, branch):
+    def app_for_branch(self, b, path):
         if not self.folder:
-            name = os.path.basename(os.path.abspath(self.root.folder))
+            name = os.path.basename(os.path.abspath(path))
         else:
             name = self.folder
-        branch_app = BranchWSGIApp(
-            branch, name, {'cachepath': sql_dir}, self.root.graph_cache)
-        return branch_app.app
-
-    def app_for_non_branch(self, environ):
-        segment = path_info_pop(environ)
-        if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
-        elif segment == '':
-            return DirectoryListing(os.path.join(self.root.folder, self.folder))
-        else:
-            relpath = os.path.join(self.folder, segment)
-            return BranchesFromFileSystemServer(relpath, self.root)
+        h = BranchWSGIApp(path, name, {'cachepath': sql_dir})
+        self.root.cache[path] = h
+        return h.app
 
     def __call__(self, environ, start_response):
         path = os.path.join(self.root.folder, self.folder)
         if not os.path.isdir(path):
             raise httpexceptions.HTTPNotFound()
+        cached = self.root.cache.get(path)
+        if cached is not None:
+            return cached.app(environ, start_response)
         try:
             b = branch.Branch.open(path)
         except errors.NotBranchError:
-            return self.app_for_non_branch(environ)(environ, start_response)
+            segment = path_info_pop(environ)
+            if segment is None:
+                raise httpexceptions.HTTPMovedPermanently(
+                    environ['SCRIPT_NAME'] + '/')
+            elif segment == '':
+                return self.directory_listing(path, environ, start_response)
+            else:
+                relpath = os.path.join(self.folder, segment)
+                return BranchesFromFileSystemServer(relpath, self.root)(
+                    environ, start_response)
         else:
-            return self.app_for_branch(b)(environ, start_response)
+            return self.app_for_branch(b, path)(environ, start_response)
 
 
 class BranchesFromFileSystemRoot(object):
-
     def __init__(self, folder):
-        self.graph_cache = lru_cache.LRUCache()
+        self.cache = {}
         self.folder = folder
-
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
         if environ['PATH_INFO'].startswith('/static/'):

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2008-06-25 09:43:29 +0000
+++ loggerhead/changecache.py	2008-06-20 04:58:42 +0000
@@ -31,6 +31,7 @@
 from loggerhead import util
 from loggerhead.lockfile import LockFile
 
+
 with_lock = util.with_lock('_lock', 'ChangeCache')
 
 SQLITE_INTERFACE = os.environ.get('SQLITE_INTERFACE', 'sqlite')
@@ -41,7 +42,8 @@
 elif SQLITE_INTERFACE == 'sqlite':
     import sqlite as dbapi2
     _param_marker = '%s'
-
+else:
+    raise AssertionError("bad sqlite interface %r!?"%SQLITE_INTERFACE)
 
 _select_stmt = ("select data from revisiondata where revid = ?"
                 ).replace('?', _param_marker)

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2008-07-09 08:39:59 +0000
+++ loggerhead/controllers/__init__.py	2008-06-20 04:58:42 +0000
@@ -16,82 +16,52 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import re
 import time
 
-from paste.request import path_info_pop, parse_querystring
+from paste.request import path_info_pop
 
 from loggerhead import util
 from loggerhead.templatefunctions import templatefunctions
 from loggerhead.zptsupport import load_template
 
-class BufferingWriter(object):
-
-    def __init__(self, writefunc, buf_limit):
-        self.bytes = 0
-        self.buf = []
-        self.buflen = 0
-        self.writefunc = writefunc
-        self.bytes_saved = 0
-        self.buf_limit = buf_limit
-
-    def flush(self):
-        chunk = ''.join(self.buf)
-        chunk = re.sub(r'\s*\n\s*', '\n', chunk)
-        chunk = re.sub(r'[ \t]+', ' ', chunk)
-        self.bytes_saved += self.buflen - len(chunk)
-        self.writefunc(chunk)
-        self.buf = []
-        self.buflen = 0
-
-    def write(self, data):
-        self.buf.append(data)
-        self.buflen += len(data)
-        self.bytes += len(data)
-        if self.buflen > self.buf_limit:
-            self.flush()
-
 class TemplatedBranchView(object):
 
     template_path = None
 
-    def __init__(self, branch, history):
+    def __init__(self, branch):
         self._branch = branch
-        self._history = history
         self.log = branch.log
 
-    def __call__(self, environ, start_response):
+    def default(self, request, response):
         z = time.time()
-        h = self._history
-        kw = dict(parse_querystring(environ))
+        h = self._branch.history
+        kw = request.GET
         util.set_context(kw)
-        args = []
-        while 1:
-            arg = path_info_pop(environ)
-            if arg is None:
-                break
-            args.append(arg)
-
-        vals = {
-            'branch': self._branch,
-            'util': util,
-            'history': h,
-            'url': self._branch.context_url,
-        }
-        vals.update(templatefunctions)
-        headers = {}
-        vals.update(self.get_values(h, args, kw, headers))
-
-        self.log.info('Getting information for %s: %r secs' % (
-            self.__class__.__name__, time.time() - z,))
-        if 'Content-Type' not in headers:
-            headers['Content-Type'] = 'text/html'
-        writer = start_response("200 OK", headers.items())
-        template = load_template(self.template_path)
-        z = time.time()
-        w = BufferingWriter(writer, 8192)
-        template.expand_into(w, **vals)
-        w.flush()
-        self.log.info('Rendering %s: %r secs, %s bytes, %s (%2.1f%%) bytes saved' % (
-            self.__class__.__name__, time.time() - z, w.bytes, w.bytes_saved, 100.0*w.bytes_saved/w.bytes))
-        return []
+
+        h._branch.lock_read()
+        try:
+            args = []
+            while 1:
+                arg = path_info_pop(request.environ)
+                if arg is None:
+                    break
+                args.append(arg)
+
+            vals = {
+                'branch': self._branch,
+                'util': util,
+                'history': h,
+                'url': self._branch.context_url,
+            }
+            vals.update(templatefunctions)
+            del response.headers['Content-Type']
+            vals.update(self.get_values(h, args, kw, response))
+
+            self.log.info('/%s: %r secs' % (
+                self.__class__.__name__, time.time() - z,))
+            if 'Content-Type' not in response.headers:
+                response.headers['Content-Type'] = 'text/html'
+            template = load_template(self.template_path)
+            template.expand_into(response, **vals)
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-06-25 09:43:29 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-06-20 04:58:42 +0000
@@ -36,7 +36,7 @@
 
     template_path = 'loggerhead.templates.annotate'
 
-    def get_values(self, h, args, kw, headers):
+    def get_values(self, h, args, kw, response):
         if len(args) > 0:
             revid = h.fix_revid(args[0])
         else:

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2008-07-09 08:39:59 +0000
+++ loggerhead/controllers/atom_ui.py	2008-06-20 04:58:42 +0000
@@ -24,14 +24,15 @@
 
     template_path = 'loggerhead.templates.atom'
 
-    def get_values(self, h, args, kw, headers):
+    def get_values(self, h, args, kw, response):
+        h = self._branch.history
 
         pagesize = int(20)#self._branch.config.get('pagesize', '20'))
 
         revid_list = h.get_file_view(h.last_revid, None)
         entries = list(h.get_changes(list(revid_list)[:pagesize]))
 
-        headers['Content-Type'] = 'application/atom+xml'
+        response.headers['Content-Type'] = 'application/atom+xml'
         return {
             'changes': entries,
             'updated': entries[0].date.isoformat() + 'Z',

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-07-09 08:39:59 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-06-20 04:58:42 +0000
@@ -27,7 +27,7 @@
 
     template_path = 'loggerhead.templates.changelog'
 
-    def get_values(self, h, args, kw, headers):
+    def get_values(self, h, args, kw, response):
         if args:
             revid = h.fix_revid(args[0])
         else:
@@ -66,7 +66,7 @@
         navigation = util.Container(
             pagesize=pagesize, revid=revid, start_revid=start_revid,
             revid_list=revid_list, filter_file_id=filter_file_id,
-            scan_url='/changes', branch=self._branch, feed=True, history=h)
+            scan_url='/changes', branch=self._branch, feed=True)
         if query is not None:
             navigation.query = query
         util.fill_in_navigation(navigation)

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2008-07-01 02:38:56 +0000
+++ loggerhead/controllers/download_ui.py	2008-06-16 08:08:33 +0000
@@ -29,39 +29,39 @@
 
 class DownloadUI (object):
 
-    def __init__(self, branch, history):
+    def __init__(self, branch):
+        # BranchView object
         self._branch = branch
-        self._history = history
         self.log = branch.log
 
-    def __call__(self, environ, start_response):
+    def default(self, request, response):
         # /download/<rev_id>/<file_id>/[filename]
         z = time.time()
-
-        h = self._history
-
-        args = []
-        while 1:
-            arg = path_info_pop(environ)
-            if arg is None:
-                break
-            args.append(arg)
-
-        if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url('../changes'))
-
-        revid = h.fix_revid(args[0])
-        file_id = args[1]
-        path, filename, content = h.get_file(file_id, revid)
-        mime_type, encoding = mimetypes.guess_type(filename)
-        if mime_type is None:
-            mime_type = 'application/octet-stream'
-
-        self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
-        headers = [
-            ('Content-Type', mime_type),
-            ('Content-Length', len(content)),
-            ('Content-Disposition', 'attachment; filename=%s'%(filename,)),
-            ]
-        start_response('200 OK', headers)
-        return [content]
+        h = self._branch.history
+
+        h._branch.lock_read()
+        try:
+            args = []
+            while 1:
+                arg = path_info_pop(request.environ)
+                if arg is None:
+                    break
+                args.append(arg)
+
+            if len(args) < 2:
+                raise httpexceptions.HTTPMovedPermanently(self._branch.url('../changes'))
+
+            revid = h.fix_revid(args[0])
+            file_id = args[1]
+            path, filename, content = h.get_file(file_id, revid)
+            mime_type, encoding = mimetypes.guess_type(filename)
+            if mime_type is None:
+                mime_type = 'application/octet-stream'
+
+            self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
+            response.headers['Content-Type'] = mime_type
+            response.headers['Content-Length'] = len(content)
+            response.headers['Content-Disposition'] = 'attachment; filename=%s'%(filename,)
+            response.write(content)
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-06-25 09:43:29 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-06-20 04:58:42 +0000
@@ -39,7 +39,7 @@
 
     template_path = 'loggerhead.templates.inventory'
 
-    def get_values(self, h, args, kw, headers):
+    def get_values(self, h, args, kw, response):
         if len(args) > 0:
             revid = h.fix_revid(args[0])
         else:

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-07-09 08:39:59 +0000
+++ loggerhead/controllers/revision_ui.py	2008-06-20 04:58:42 +0000
@@ -30,7 +30,7 @@
 
     template_path = 'loggerhead.templates.revision'
 
-    def get_values(self, h, args, kw, headers):
+    def get_values(self, h, args, kw, response):
 
         if len(args) > 0:
             revid = h.fix_revid(args[0])
@@ -52,7 +52,7 @@
         navigation = util.Container(
             revid_list=revid_list, revid=revid, start_revid=start_revid,
             filter_file_id=filter_file_id, pagesize=1,
-            scan_url='/revision', branch=self._branch, feed=True, history=h)
+            scan_url='/revision', branch=self._branch, feed=True)
         if query is not None:
             navigation.query = query
         util.fill_in_navigation(navigation)

=== modified file 'loggerhead/controllers/search_ui.py'
--- loggerhead/controllers/search_ui.py	2008-07-09 13:41:07 +0000
+++ loggerhead/controllers/search_ui.py	2008-06-20 05:50:14 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2008  Canonical Ltd
-# Authored by Martin Albisetti <argentina@gmail.com>
+# Copyright (C) 2008  Martin Albisetti <argentina@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,10 +41,6 @@
         query = kwargs['query']
         if len(query) > 0:
             terms = search.search_revisions(h._branch, query, True)
-            if terms is not None:
-                terms = [term[0] for term in terms]
-            else:
-                # Should show a 'search is not available' etc box.
-                terms = []
+            terms = [term[0] for term in terms] 
 
         return {'terms':terms}

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-07-17 03:40:03 +0000
+++ loggerhead/history.py	2008-06-24 17:12:45 +0000
@@ -37,7 +37,7 @@
 
 from loggerhead import search
 from loggerhead import util
-from loggerhead.wholehistory import compute_whole_history_data
+from loggerhead.util import decorator
 
 import bzrlib
 import bzrlib.branch
@@ -48,6 +48,22 @@
 import bzrlib.tsort
 import bzrlib.ui
 
+with_branch_lock = util.with_lock('_lock', 'branch')
+
+
+@decorator
+def with_bzrlib_read_lock(unbound):
+    def bzrlib_read_locked(self, *args, **kw):
+        #self.log.debug('-> %r bzr lock', id(threading.currentThread()))
+        self._branch.repository.lock_read()
+        try:
+            return unbound(self, *args, **kw)
+        finally:
+            self._branch.repository.unlock()
+            #self.log.debug('<- %r bzr lock', id(threading.currentThread()))
+    return bzrlib_read_locked
+
+
 # bzrlib's UIFactory is not thread-safe
 uihack = threading.local()
 
@@ -172,33 +188,88 @@
 
 
 class History (object):
-    """Decorate a branch to provide information for rendering.
-
-    History objects are expected to be short lived -- when serving a request
-    for a particular branch, open it, read-lock it, wrap a History object
-    around it, serve the request, throw the History object away, unlock the
-    branch and throw it away.
-
-    :ivar _file_change_cache: xx
-    """
-
-    def __init__(self, branch, whole_history_data_cache):
-        assert branch.is_locked(), (
-            "Can only construct a History object with a read-locked branch.")
+
+    def __init__(self):
         self._file_change_cache = None
+        self._lock = threading.RLock()
+
+    @classmethod
+    def from_branch(cls, branch):
+        z = time.time()
+        self = cls()
         self._branch = branch
-        self.log = logging.getLogger('loggerhead.%s' % (branch.nick,))
-
-        self.last_revid = branch.last_revision()
-
-        whole_history_data = whole_history_data_cache.get(self.last_revid)
-        if whole_history_data is None:
-            whole_history_data = compute_whole_history_data(branch)
-            whole_history_data_cache[self.last_revid] = whole_history_data
-
-        (self._revision_graph, self._full_history, self._revision_info,
-         self._revno_revid, self._merge_sort, self._where_merged
-         ) = whole_history_data
+        self._last_revid = self._branch.last_revision()
+
+        self.log = logging.getLogger('loggerhead.%s' % (self._branch.nick,))
+
+        graph = branch.repository.get_graph()
+        parent_map = dict(((key, value) for key, value in
+             graph.iter_ancestry([self._last_revid]) if value is not None))
+
+        self._revision_graph = self._strip_NULL_ghosts(parent_map)
+        self._full_history = []
+        self._revision_info = {}
+        self._revno_revid = {}
+        if bzrlib.revision.is_null(self._last_revid):
+            self._merge_sort = []
+        else:
+            self._merge_sort = bzrlib.tsort.merge_sort(
+                self._revision_graph, self._last_revid, generate_revno=True)
+
+        for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
+            self._full_history.append(revid)
+            revno_str = '.'.join(str(n) for n in revno)
+            self._revno_revid[revno_str] = revid
+            self._revision_info[revid] = (
+                seq, revid, merge_depth, revno_str, end_of_merge)
+
+        # cache merge info
+        self._where_merged = {}
+
+        for revid in self._revision_graph.keys():
+            if self._revision_info[revid][2] == 0:
+                continue
+            for parent in self._revision_graph[revid]:
+                self._where_merged.setdefault(parent, set()).add(revid)
+
+        self.log.info('built revision graph cache: %r secs' % (time.time() - z,))
+        return self
+
+    @staticmethod
+    def _strip_NULL_ghosts(revision_graph):
+        """
+        Copied over from bzrlib meant as a temporary workaround deprecated 
+        methods.
+        """
+
+        # Filter ghosts, and null:
+        if bzrlib.revision.NULL_REVISION in revision_graph:
+            del revision_graph[bzrlib.revision.NULL_REVISION]
+        for key, parents in revision_graph.items():
+            revision_graph[key] = tuple(parent for parent in parents if parent
+                in revision_graph)
+        return revision_graph
+
+    @classmethod
+    def from_folder(cls, path):
+        b = bzrlib.branch.Branch.open(path)
+        b.lock_read()
+        try:
+            return cls.from_branch(b)
+        finally:
+            b.unlock()
+
+    @with_branch_lock
+    def out_of_date(self):
+        # the branch may have been upgraded on disk, in which case we're stale.
+        newly_opened = bzrlib.branch.Branch.open(self._branch.base)
+        if self._branch.__class__ is not \
+               newly_opened.__class__:
+            return True
+        if self._branch.repository.__class__ is not \
+               newly_opened.repository.__class__:
+            return True
+        return self._branch.last_revision() != self._last_revid
 
     def use_file_cache(self, cache):
         self._file_change_cache = cache
@@ -207,6 +278,9 @@
     def has_revisions(self):
         return not bzrlib.revision.is_null(self.last_revid)
 
+    last_revid = property(lambda self: self._last_revid, None, None)
+
+    @with_branch_lock
     def get_config(self):
         return self._branch.get_config()
 
@@ -217,6 +291,9 @@
         seq, revid, merge_depth, revno_str, end_of_merge = self._revision_info[revid]
         return revno_str
 
+    def get_revision_history(self):
+        return self._full_history
+
     def get_revids_from(self, revid_list, start_revid):
         """
         Yield the mainline (wrt start_revid) revisions that merged each
@@ -244,15 +321,18 @@
                 return
             revid = parents[0]
 
+    @with_branch_lock
     def get_short_revision_history_by_fileid(self, file_id):
         # wow.  is this really the only way we can get this list?  by
         # man-handling the weave store directly? :-0
         # FIXME: would be awesome if we could get, for a folder, the list of
         # revisions where items within that folder changed.
-        possible_keys = [(file_id, revid) for revid in self._full_history]
-        existing_keys = self._branch.repository.texts.get_parent_map(possible_keys)
-        return [revid for _, revid in existing_keys.iterkeys()]
+        w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
+        w_revids = w.versions()
+        revids = [r for r in self._full_history if r in w_revids]
+        return revids
 
+    @with_branch_lock
     def get_revision_history_since(self, revid_list, date):
         # if a user asks for revisions starting at 01-sep, they mean inclusive,
         # so start at midnight on 02-sep.
@@ -266,6 +346,7 @@
         index = -index
         return revid_list[index:]
 
+    @with_branch_lock
     def get_search_revid_list(self, query, revid_list):
         """
         given a "quick-search" query, try a few obvious possible meanings:
@@ -304,7 +385,7 @@
         if date is not None:
             if revid_list is None:
                 # if no limit to the query was given, search only the direct-parent path.
-                revid_list = list(self.get_revids_from(None, self.last_revid))
+                revid_list = list(self.get_revids_from(None, self._last_revid))
             return self.get_revision_history_since(revid_list, date)
 
     revno_re = re.compile(r'^[\d\.]+$')
@@ -320,11 +401,12 @@
         if revid is None:
             return revid
         if revid == 'head:':
-            return self.last_revid
+            return self._last_revid
         if self.revno_re.match(revid):
             revid = self._revno_revid[revid]
         return revid
 
+    @with_branch_lock
     def get_file_view(self, revid, file_id):
         """
         Given a revid and optional path, return a (revlist, revid) for
@@ -334,7 +416,7 @@
         If file_id is None, the entire revision history is the list scope.
         """
         if revid is None:
-            revid = self.last_revid
+            revid = self._last_revid
         if file_id is not None:
             # since revid is 'start_revid', possibly should start the path
             # tracing from revid... FIXME
@@ -344,6 +426,7 @@
             revlist = list(self.get_revids_from(None, revid))
         return revlist
 
+    @with_branch_lock
     def get_view(self, revid, start_revid, file_id, query=None):
         """
         use the URL parameters (revid, start_revid, file_id, and query) to
@@ -369,7 +452,7 @@
         contain vital context for future url navigation.
         """
         if start_revid is None:
-            start_revid = self.last_revid
+            start_revid = self._last_revid
 
         if query is None:
             revid_list = self.get_file_view(start_revid, file_id)
@@ -393,14 +476,13 @@
                 revid = revid_list[0]
             return revid, start_revid, revid_list
         else:
-            # XXX: This should return a message saying that the search could
-            # not be completed due to either missing the plugin or missing a
-            # search index.
             return None, None, []
 
+    @with_branch_lock
     def get_inventory(self, revid):
         return self._branch.repository.get_revision_inventory(revid)
 
+    @with_branch_lock
     def get_path(self, revid, file_id):
         if (file_id is None) or (file_id == ''):
             return ''
@@ -409,6 +491,7 @@
             path = '/' + path
         return path
 
+    @with_branch_lock
     def get_file_id(self, revid, path):
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
@@ -487,6 +570,7 @@
                 else:
                     p.branch_nick = '(missing)'
 
+    @with_branch_lock
     def get_changes(self, revid_list):
         """Return a list of changes objects for the given revids.
 
@@ -513,6 +597,8 @@
 
         return changes
 
+    @with_branch_lock
+    @with_bzrlib_read_lock
     def get_changes_uncached(self, revid_list):
         # FIXME: deprecated method in getting a null revision
         revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
@@ -583,6 +669,7 @@
 
         return [self.parse_delta(delta) for delta in delta_list]
 
+    @with_branch_lock
     def get_file_changes(self, entries):
         if self._file_change_cache is None:
             return self.get_file_changes_uncached(entries)
@@ -595,6 +682,7 @@
         for entry, changes in zip(entries, changes_list):
             entry.changes = changes
 
+    @with_branch_lock
     def get_change_with_diff(self, revid, compare_revid=None):
         change = self.get_changes([revid])[0]
 
@@ -613,6 +701,7 @@
 
         return change
 
+    @with_branch_lock
     def get_file(self, file_id, revid):
         "returns (path, filename, data)"
         inv = self.get_inventory(revid)
@@ -745,6 +834,7 @@
             for m in change.changes.modified:
                 m.sbs_chunks = _make_side_by_side(m.chunks)
 
+    @with_branch_lock
     def get_filelist(self, inv, file_id, sort_type=None):
         """
         return the list of all files (and their attributes) within a given
@@ -797,6 +887,7 @@
 
     _BADCHARS_RE = re.compile(ur'[\x00-\x08\x0b\x0e-\x1f]')
 
+    @with_branch_lock
     def annotate_file(self, file_id, revid):
         z = time.time()
         lineno = 1

=== modified file 'loggerhead/search.py'
--- loggerhead/search.py	2008-07-09 13:41:07 +0000
+++ loggerhead/search.py	2008-06-21 00:48:49 +0000
@@ -1,6 +1,4 @@
-# Copyright (C) 2008  Canonical Ltd
-# Authored by Martin Albisetti <argentina@gmail.com>
-#
+# Copyright (C) 2008  Martin Albisetti <argentina@gmail.com>
 # Copyright (C) 2008  Robert Collins
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,12 +17,11 @@
 #
 import sets
 import os
-try:
-    from bzrlib.plugins.search import errors
-    from bzrlib.plugins.search import index as _mod_index
-    from bzrlib.plugins.search.index import FileTextHit, RevisionHit
-except ImportError:
-    _mod_index = None
+from bzrlib.plugins.search import errors
+from bzrlib.plugins.search import index as _mod_index
+from bzrlib.plugins.search.index import FileTextHit, RevisionHit
+from bzrlib.plugin import load_plugins
+load_plugins()
 
 def search_revisions(branch, query_list, suggest=False):
     """
@@ -36,12 +33,7 @@
     param suggest: Optional flag to request suggestions instead of results
     return: A list for results, either revision ids or terms
     """
-    if _mod_index is None:
-        return None # None indicates could-not-search
-    try:
-        index = _mod_index.open_index_branch(branch)
-    except errors.NoSearchIndex:
-        return None # None indicates could-not-search
+    index = _mod_index.open_index_branch(branch)
     query = query_list.split(' ')
     query = [(term,) for term in query]
     revid_list = []

=== modified file 'loggerhead/static/css/zptstyle.css'
--- loggerhead/static/css/zptstyle.css	2008-07-17 03:53:19 +0000
+++ loggerhead/static/css/zptstyle.css	2008-06-20 04:16:25 +0000
@@ -673,7 +673,7 @@
 #search_terms {
     display:none;
     border:solid 1px black;
-    width:180px;
+    width:300px;
     overflow:auto;
     background: #fff;
 }

=== modified file 'loggerhead/static/images/feed-icon-16x16.gif' (properties changed: -x to +x)
=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2008-07-17 04:07:21 +0000
+++ loggerhead/static/javascript/custom.js	2008-06-24 17:10:42 +0000
@@ -1,5 +1,4 @@
 var global_timeout_id = null;
-var global_search_request = null;
 
 window.addEvent('domready', function() 
 {
@@ -8,30 +7,17 @@
     {
         keyup: function()
         {
-            if($('q').value == '')
-            {
-                $('search_terms').setStyle('display','none');
-            }
-            else
-            {
-	            if (null != global_timeout_id)
-	            {
-	                clearTimeout(global_timeout_id);
-	            }
-                    global_timeout_id = setTimeout('$("q").fireEvent("search",$("q").value)',200);
-            }
+	    if (null != global_timeout_id)
+	    {
+	        clearTimeout(global_timeout_id);
+	    }
+            global_timeout_id = setTimeout('$("q").fireEvent("search",$("q").value)',200);
         },
 
         search: function(query)
         {
-            url = global_path + 'search?query=' + query;
+            url = 'search?query=' + query;
 	
-            if ($defined(global_search_request))
-            {
-                global_search_request.cancel();
-            }
-            global_search_request = new Request({'url':url,'method':'get','onComplete':cool_search});
-            global_search_request.send('');
 	        var posicion = search_box.getPosition();
 	        var size     = search_box.getSize();
 
@@ -58,8 +44,3 @@
 	$('search_terms').setStyle('left',posicion.x);
 	$('search_terms').setStyle('top',posicion.y + size.y);
 }
-
-function hide_search()
-{
-    hide_div = setTimeout("$('search_terms').setStyle('display','none')", 300);
-}

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2008-06-26 20:42:15 +0000
+++ loggerhead/templates/atom.pt	2008-06-11 04:41:08 +0000
@@ -4,7 +4,7 @@
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
   <updated tal:content="updated">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
+  <id tal:content="python:branch.url(['atom'])">url</id>
   <link rel="self" tal:attributes="href python:branch.url([''])" />
 
   <entry tal:repeat="entry changes">
@@ -14,7 +14,7 @@
     <updated tal:content="entry/date/isoformat">
     Z
     </updated>
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <id tal:content="python:branch.url(['revision', entry.revno])">
       ID
     </id>
     <author>
@@ -26,6 +26,6 @@
       comment
     </content>
     <link rel="alternate"
-          tal:attributes="href python:branch.url(['/revision', entry.revno])" />
+          tal:attributes="href python:branch.url(['revision', entry.revno])" />
   </entry>
 </feed>

=== modified file 'loggerhead/templates/browse.pt'
--- loggerhead/templates/browse.pt	2008-07-01 03:28:31 +0000
+++ loggerhead/templates/browse.pt	2008-06-18 04:16:21 +0000
@@ -45,7 +45,7 @@
               </th>
               <th />
             </tr>
-            <tal:block repeat="view python:views_by_project[project]">
+            <tal:block repeat="view project/views">
               <tr>
                 <td class="name">
                   <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name)"
@@ -58,7 +58,7 @@
                   ${view.description}
                 </td>
                 <td class="last-update"
-                    tal:content="python:view.get_history().has_revisions and util._approximatedate(view.last_updated()) or '(no revisions)'"></td>
+                    tal:content="python:view.history.has_revisions and util._approximatedate(view.last_updated()) or '(no revisions)'"></td>
                 <td class="feed">
                   <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name + '/atom')"
                      title="atom feed">

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2008-06-25 09:20:55 +0000
+++ loggerhead/templates/changelog.pt	2008-06-20 04:58:42 +0000
@@ -9,11 +9,18 @@
       <metal:block fill-slot="header_extras">
         <link rel="alternate" type="application/atom+xml"
               tal:attributes="href python:url(['/atom']);
-                              title string:RSS feed for ${branch/friendly_name}"/>
+                              title string:RSS feed for ${branch/friendly_name}"
+              />
 
         <script type="text/javascript"
+                tal:attributes="src python:branch.static_url('/static/javascript/mootools.js')">
+        </script>
+        <script type="text/javascript"
                 tal:attributes="src python:branch.static_url('/static/javascript/collapse.js')">
         </script>
+        <script type="text/javascript"
+                tal:attributes="src python:branch.static_url('/static/javascript/custom.js')">
+        </script>
       </metal:block>
     </head>
 

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2008-07-17 03:53:19 +0000
+++ loggerhead/templates/macros.pt	2008-06-20 05:17:46 +0000
@@ -11,19 +11,11 @@
     <link rel="stylesheet"
           tal:attributes="href python:branch.static_url('/static/css/zptstyle.css')" />
 
-    <script type="text/javascript"
-            tal:attributes="src python:branch.static_url('/static/javascript/mootools.js')">
-    </script>
-    <script type="text/javascript"
-            tal:attributes="src python:branch.static_url('/static/javascript/custom.js')">
-    </script>
     <metal:block metal:define-slot="header_extras" />
   </head>
 
   <body tal:attributes="onload onload|nothing">
-      <script language="javascript">
-          var global_path = <tal:block content="python:'\''+branch.url('/')+'\''" />;
-      </script>
+
     <div class="bar" tal:condition="navigation|nothing">
       <form tal:attributes="action python:branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None), file_id=getattr(navigation, 'file_id', None))">
         <table>
@@ -33,7 +25,7 @@
                 <a tal:attributes="href python:url('/changes', clear=1)"> changes </a>
                 <a tal:attributes="href python:url('/files', clear=1)"> files </a>
                 <span class="search">
-                    search: <input type="text" name="q" id="q" autocomplete="off" onblur="hide_search();" />
+                    search: <input type="text" name="q" id="q" autocomplete="off" />
                 </span>
               </span>
             </td>

=== modified file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	2008-06-27 04:32:15 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2007-09-05 18:40:01 +0000
@@ -24,7 +24,11 @@
     <td class="files"
         tal:define="filename python:removed[0];
                     file_id  python:removed[1]">
-      <tal:block content="filename" />
+      <a tal:attributes="href  python:url(['/annotate', entry.revid], file_id=file_id);
+                         title string:Annotate ${filename}"
+         tal:content="filename" class="filename">
+        filename
+      </a>
     </td>
   </tr>
   <tr tal:repeat="renamed entry/changes/renamed">

=== modified file 'loggerhead/templates/search.pt'
--- loggerhead/templates/search.pt	2008-07-17 04:12:49 +0000
+++ loggerhead/templates/search.pt	2008-06-20 05:50:14 +0000
@@ -1,8 +1,5 @@
-<ul style="list-style:none; list-style-display:inline; padding:2px; max-height:450px;">
-<tal:block repeat="term terms" condition="python: len(terms) > 0">
-    <li><a tal:attributes="href python:url(['/changes'], q=term)" tal:content="term"></a></li>
-</tal:block>
-<tal:block condition="python: len(terms) == 0">
-    <li>No results found.</li>
+<ul style="list-style:none;">
+<tal:block repeat="term terms">
+<li><a tal:attributes="href python:url(['/changes'], q=term)" tal:content="term"></a></li>
 </tal:block>
 </ul>

=== modified file 'loggerhead/tests/test_filechangecache.py'
--- loggerhead/tests/test_filechangecache.py	2008-06-25 07:03:06 +0000
+++ loggerhead/tests/test_filechangecache.py	2007-09-10 12:51:42 +0000
@@ -5,12 +5,12 @@
 
 from loggerhead.changecache import FileChangeCache
 
-class MockEntry(object):
+class MockEntry:
     def __init__(self, revid):
         self.revid = revid
 
 
-class MockHistory(object):
+class MockHistory:
     def __init__(self):
         self.fetched_revids = set()
     def get_file_changes_uncached(self, entries):

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2008-07-01 04:01:31 +0000
+++ loggerhead/tests/test_simple.py	2008-06-18 12:27:04 +0000
@@ -51,8 +51,11 @@
             folder = '%(branch)s'
     """
 
+    def makeHistory(self):
+        return History.from_folder(self.bzrbranch)
+
     def setUpLoggerhead(self):
-        app = TestApp(BranchWSGIApp(self.branch).app)
+        app = TestApp(BranchWSGIApp(self.bzrbranch).app)
         return app
 
     def tearDown(self):

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-07-01 00:10:47 +0000
+++ loggerhead/util.py	2008-06-20 02:20:05 +0000
@@ -27,9 +27,12 @@
 import datetime
 import logging
 import re
+import sha
 import struct
+import sys
 import threading
 import time
+import traceback
 
 
 log = logging.getLogger("loggerhead.controllers")
@@ -139,12 +142,28 @@
         return out
 
 
+def clean_revid(revid):
+    if revid == 'missing':
+        return revid
+    return sha.new(revid).hexdigest()
+
+
+def obfuscate(text):
+    return ''.join([ '&#%d;' % ord(c) for c in text ])
+
+
 def trunc(text, limit=10):
     if len(text) <= limit:
         return text
     return text[:limit] + '...'
 
 
+def to_utf8(s):
+    if isinstance(s, unicode):
+        return s.encode('utf-8')
+    return s
+
+
 STANDARD_PATTERN = re.compile(r'^(.*?)\s*<(.*?)>\s*$')
 EMAIL_PATTERN = re.compile(r'[-\w\d\+_!%\.]+@[-\w\d\+_!%\.]+')
 
@@ -168,6 +187,40 @@
     return '%s at %s' % (username, domains[0])
 
 
+def triple_factors(min_value=1):
+    factors = (1, 3)
+    index = 0
+    n = 1
+    while True:
+        if n >= min_value:
+            yield n * factors[index]
+        index += 1
+        if index >= len(factors):
+            index = 0
+            n *= 10
+
+
+def scan_range(pos, max, pagesize=1):
+    """
+    given a position in a maximum range, return a list of negative and positive
+    jump factors for an hgweb-style triple-factor geometric scan.
+
+    for example, with pos=20 and max=500, the range would be:
+    [ -10, -3, -1, 1, 3, 10, 30, 100, 300 ]
+
+    i admit this is a very strange way of jumping through revisions.  i didn't
+    invent it. :)
+    """
+    out = []
+    for n in triple_factors(pagesize + 1):
+        if n > max:
+            return out
+        if pos + n < max:
+            out.append(n)
+        if pos - n >= 0:
+            out.insert(0, -n)
+
+
 # only do this if unicode turns out to be a problem
 #_BADCHARS_RE = re.compile(ur'[\u007f-\uffff]')
 
@@ -213,6 +266,15 @@
     return '-rw-r--r--'
 
 
+def if_present(format, value):
+    """
+    format a value using a format string, if the value exists and is not None.
+    """
+    if value is None:
+        return ''
+    return format % value
+
+
 def b64(s):
     s = base64.encodestring(s).replace('\n', '')
     while (len(s) > 0) and (s[-1] == '='):
@@ -294,11 +356,11 @@
 
     navigation.prev_page_revid = get_offset(-1 * navigation.pagesize)
     navigation.next_page_revid = get_offset(1 * navigation.pagesize)
-    prev_page_revno = navigation.history.get_revno(
+    prev_page_revno = navigation.branch.history.get_revno(
             navigation.prev_page_revid)
-    next_page_revno = navigation.history.get_revno(
+    next_page_revno = navigation.branch.history.get_revno(
             navigation.next_page_revid)
-    start_revno = navigation.history.get_revno(navigation.start_revid)
+    start_revno = navigation.branch._history.get_revno(navigation.start_revid)
 
     params = { 'filter_file_id': navigation.filter_file_id }
     if getattr(navigation, 'query', None) is not None:
@@ -315,6 +377,11 @@
             [navigation.scan_url, next_page_revno], **params)
 
 
+def log_exception(log):
+    for line in ''.join(traceback.format_exception(*sys.exc_info())).split('\n'):
+        log.debug(line)
+
+
 def decorator(unbound):
     def new_decorator(f):
         g = unbound(f)
@@ -345,6 +412,22 @@
 
 
 @decorator
+def strip_whitespace(f):
+    def _f(*a, **kw):
+        out = f(*a, **kw)
+        orig_len = len(out)
+        out = re.sub(r'\n\s+', '\n', out)
+        out = re.sub(r'[ \t]+', ' ', out)
+        out = re.sub(r'\s+\n', '\n', out)
+        new_len = len(out)
+        log.debug('Saved %sB (%d%%) by stripping whitespace.',
+                  human_size(orig_len - new_len),
+                  round(100.0 - float(new_len) * 100.0 / float(orig_len)))
+        return out
+    return _f
+
+
+@decorator
 def lsprof(f):
     def _f(*a, **kw):
         from loggerhead.lsprof import profile

=== removed file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2008-07-01 02:24:16 +0000
+++ loggerhead/wholehistory.py	1970-01-01 00:00:00 +0000
@@ -1,63 +0,0 @@
-# Cache the whole history data needed by loggerhead about a branch.
-
-import logging
-import time
-
-from bzrlib.revision import is_null, NULL_REVISION
-from bzrlib.tsort import merge_sort
-
-
-def _strip_NULL_ghosts(revision_graph):
-    """
-    Copied over from bzrlib meant as a temporary workaround for
-    deprecated methods.
-    """
-    # Filter ghosts, and null:
-    if NULL_REVISION in revision_graph:
-        del revision_graph[NULL_REVISION]
-    for key, parents in revision_graph.items():
-        revision_graph[key] = tuple(parent for parent in parents if parent
-            in revision_graph)
-    return revision_graph
-
-
-def compute_whole_history_data(branch):
-    z = time.time()
-
-    last_revid = branch.last_revision()
-
-    log = logging.getLogger('loggerhead.%s' % (branch.nick,))
-
-    graph = branch.repository.get_graph()
-    parent_map = dict(((key, value) for key, value in
-         graph.iter_ancestry([last_revid]) if value is not None))
-
-    _revision_graph = _strip_NULL_ghosts(parent_map)
-    _full_history = []
-    _revision_info = {}
-    _revno_revid = {}
-    if is_null(last_revid):
-        _merge_sort = []
-    else:
-        _merge_sort = merge_sort(
-            _revision_graph, last_revid, generate_revno=True)
-
-    for (seq, revid, merge_depth, revno, end_of_merge) in _merge_sort:
-        _full_history.append(revid)
-        revno_str = '.'.join(str(n) for n in revno)
-        _revno_revid[revno_str] = revid
-        _revision_info[revid] = (
-            seq, revid, merge_depth, revno_str, end_of_merge)
-
-    _where_merged = {}
-
-    for revid in _revision_graph.keys():
-        if _revision_info[revid][2] == 0:
-            continue
-        for parent in _revision_graph[revid]:
-            _where_merged.setdefault(parent, set()).add(revid)
-
-    log.info('built revision graph cache: %r secs' % (time.time() - z,))
-
-    return (_revision_graph, _full_history, _revision_info,
-            _revno_revid, _merge_sort, _where_merged)

=== modified file 'serve-branches.py'
--- serve-branches.py	2008-06-24 23:20:59 +0000
+++ serve-branches.py	2008-06-23 02:23:46 +0000
@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-import logging
 import sys
 
 from paste import httpserver
@@ -8,10 +7,6 @@
 
 from loggerhead.apps.filesystem import BranchesFromFileSystemRoot
 
-
-logging.basicConfig()
-logging.getLogger().setLevel(logging.DEBUG)
-
 if len(sys.argv) > 1:
     path = sys.argv[1]
 else:

=== modified file 'setup.py'
--- setup.py	2008-07-25 20:56:12 +0000
+++ setup.py	2008-02-28 04:34:51 +0000
@@ -23,7 +23,7 @@
 # for some reason, distutils can't find bzr.
 #        "bzr >= 0.13",
     ],
-    scripts = ["start-loggerhead", "stop-loggerhead"],
+    scripts = ["start-loggerhead.py", "stop-loggerhead.py"],
     zip_safe=False,
     packages=find_packages(),
     package_data = find_package_data(where='loggerhead',

=== removed file 'start-loggerhead.1'
--- start-loggerhead.1	2008-07-26 14:09:59 +0000
+++ start-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,29 +0,0 @@
-.TH START-LOGGERHEAD "1" "July 2008" "start-loggerhead 1.2.1" "User Commands"
-.SH NAME
-start-loggerhead \- start a loggerhead server
-.SH SYNOPSIS
-.B start-loggerhead
-[\fIoptions\fR]
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.PP
-The start-loggerhead command starts a new standalone loggerhead server.
-.SH OPTIONS
-.TP
-\fB\-\-version\fR
-show program's version number and exit
-.TP
-\fB\-h\fR, \fB\-\-help\fR
-show this help message and exit
-.TP
-\fB\-f\fR, \fB\-\-foreground\fR
-run in the foreground; don't daemonize
-.TP
-\fB\-C\fR, \fB\-\-check\fR
-only start if not already running (useful for cron jobs)
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

=== renamed file 'start-loggerhead' => 'start-loggerhead.py'
--- start-loggerhead	2008-07-25 20:56:12 +0000
+++ start-loggerhead.py	2008-06-24 17:12:45 +0000
@@ -50,9 +50,9 @@
     access_log.setLevel(logging.INFO)
     access_log.setFormatter(f)
 
-    logging.getLogger('').setLevel(logging.DEBUG)
     logging.getLogger('').addHandler(debug_log)
-    logging.getLogger('wsgi').addHandler(access_log)
+    logging.getLogger('turbogears.access').addHandler(access_log)
+    logging.getLogger('turbogears.controllers').setLevel(logging.INFO)
 
     if foreground:
         logging.getLogger('').addHandler(stdout_log)

=== removed file 'stop-loggerhead.1'
--- stop-loggerhead.1	2008-07-26 14:09:59 +0000
+++ stop-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,15 +0,0 @@
-.TH STOP-LOGGERHEAD "1" "July 2008" "stop-loggerhead 1.2.1" "User Commands"
-.SH NAME
-stop-loggerhead \- stop a loggerhead server
-.SH SYNOPSIS
-.B stop-loggerhead
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.PP
-The stop-loggerhead command stops the currently running loggerhead server.
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

=== renamed file 'stop-loggerhead' => 'stop-loggerhead.py'
